/* 
 *  EEPROM read/writes on ESP8266 work different from normal Arduino
 *  ESP8266 does not have EEPRO, so it is emulated in flash
 *  EEPROM.begin(512) reads EEPROM content into RAM
 *  EEPROM.read and write reads from or writes to this memory
 *  An EEPROM.commit() is necessary to write all 512 bytes to ESP8266 flash
 *  EEPROM.end does also EEPROM.commit() and clears the 512 bytes in RAM
 */
   struct freq_cal_t {
    int     cal_frequency;         // in MHz
    int     cal_slope;             // in mv*10/dB (enables storing float as integer)
    int     cal_intercept;         // in dBm*10 (enables storing float as integer)
    int     cal_CRC;
  }; //calib_freq[7];

  union freq_cal_union{
    freq_cal_t calib_freq[7];
    byte cal_freq_bytearray[112]; // ESP8266 has 4 bytes per int
  }fr_union;

//--------------------------------------------------------------------------------
bool write_EEPROM(int freq_n, int freq, float slope, float intercept)
{
  int i;
  if(freq_n<0 || freq_n>6) return 0;
  
  float t_slope=slope*10000;  //convert 0.0245 V to 245 (mV*10)
  float t_intercept = intercept*10;

  fr_union.calib_freq[freq_n].cal_frequency = freq;
  fr_union.calib_freq[freq_n].cal_slope = t_slope;
  fr_union.calib_freq[freq_n].cal_intercept = t_intercept; // convert intercept
  fr_union.calib_freq[freq_n].cal_CRC = CRC8(&fr_union.cal_freq_bytearray[freq_n*16], 12);

  for(i=freq_n*16;i < freq_n*16+16; i++){
    EEPROM.write(i, fr_union.cal_freq_bytearray[i]);
  }
  
  bool write_result = EEPROM.commit(); //returns true if succesful
  return write_result;
}

//--------------------------------------------------------------------------------

bool read_EEPROM(int freq_n){
  int i;
  byte CRC;
  
  if(freq_n<0 || freq_n>6) return 0;
  
  for(i=freq_n*16; i < freq_n*16+16; i++){
     fr_union.cal_freq_bytearray[i] = EEPROM.read(i);
     
  }
  
  CRC = CRC8(&fr_union.cal_freq_bytearray[freq_n*16], 12);
  
  if(CRC==0 && fr_union.calib_freq[freq_n].cal_frequency==0) return 0; // CRC 0 and frequency 0, no data
  
  if(CRC==fr_union.calib_freq[freq_n].cal_CRC)
  {
    frequency = fr_union.calib_freq[freq_n].cal_frequency;
    slope = float(fr_union.calib_freq[freq_n].cal_slope)/10000;
    intercept = float(fr_union.calib_freq[freq_n].cal_intercept)/10;
    return 1;
  }
  else return 0;

}

//--------------------------------------------------------------------------------
bool erase_EEPROM(int freq_n)
{
  byte i;
    if(freq_n<0 || freq_n>6) return 0;
    
    for(i=freq_n*16; i < freq_n*16+16; i++){
    EEPROM.write(i, 0);
  }
  
  return EEPROM.commit();

}

//--------------------------------------------------------------------------------
//CRC-8 - based on the CRC8 formulas by Dallas/Maxim
//code released under the therms of the GNU GPL 3.0 license
byte CRC8(byte *data, byte len) {
  byte crc = 0x00;
  while (len--) {
    byte extract = *data++;
    for (byte tempI = 8; tempI; tempI--) {
      byte sum = (crc ^ extract) & 0x01;
      crc >>= 1;
      if (sum) {
        crc ^= 0x8C;
      }
      extract >>= 1;
    }
  }
  return crc;
}
